package com.jeeplus.finance.reimbursementApproval.approvalInfo.controller;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.common.excel.ExcelOptions;
import com.jeeplus.common.excel.annotation.ExportMode;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.finance.invoice.util.EasyPoiUtil;
import com.jeeplus.finance.projectRecords.domain.CwProjectRecords;
import com.jeeplus.finance.projectRecords.service.dto.CwProjectRecordsDTO;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.domain.CwReimbursementFileSupplement;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.domain.CwReimbursementInfo;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.domain.CwReimbursementInfoUpHi;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.domain.ProgramProjectListInfo;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.service.CwReimbursementInfoService;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.service.dto.*;
import com.jeeplus.flowable.feign.IFlowableApi;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.logging.constant.enums.LogTypeEnum;
import com.jeeplus.sys.feign.IDictApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.ResponseEntity;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: 王强
 * @create: 2022-11-24 16:44
 **/
@RestController
@Api(tags ="财务-报销申请")
@RequestMapping(value = "/reimbursementApproval/info")
public class CwReimbursementInfoController {

    @Resource
    private CwReimbursementInfoService service;
    @Resource
    private IFlowableApi flowTaskService;


    /**
     * 报告号查询
     * @param dto
     * @return
     */
    @ApiOperation(value = "报告号查询")
    @GetMapping("/reportNoList")
    public ResponseEntity<IPage<ReportNoDto>> reportNoList(Page<ReportNoDto> page, ReportNoDto dto) throws Exception{
        IPage<ReportNoDto> iPage = service.getReportNoList(page, dto);
        return ResponseEntity.ok(iPage);
    }

    /**
     * 列表查询
     * @param dto/reimbursementApproval/type/bxList
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<IPage<RetureListDto>> list(Page<RetureListDto> page, QueryListDto dto) throws Exception{
        IPage<RetureListDto> iPage = service.list(page, dto);
        iPage.getRecords().stream().forEach(i -> {
            // 报销审批
            if (StringUtils.isNotBlank(i.getTaskId()) && StringUtils.isNotBlank(i.getType())) {
                if ("2".equals(i.getType())) { // “审核中”的数据要获取数据审核人
                    i.setAuditUserIds(flowTaskService.getTaskAuditUsers(i.getTaskId()));  // 获取数据审核人
                    String currentTaskName = flowTaskService.getCurrentTaskName(i.getProcInsId());
                    if (com.jeeplus.utils.StringUtils.isNotBlank(currentTaskName) && !currentTaskName.contains("综合管理部主任") && !currentTaskName.contains("部门主任")){
                        i.setApplyFile(true);
                    }
                }
            }
            //文件补充
            if (StringUtils.isNotBlank(i.getTaskIdFile()) && StringUtils.isNotBlank(i.getFileStatus())) {
                if ("2".equals(i.getFileStatus()) || "4".equals(i.getFileStatus())) { // “审核中”的数据要获取数据审核人
                    i.setAuditFileUserIds(flowTaskService.getTaskAuditUsers(i.getTaskIdFile()));  // 获取数据审核人
                }
            }
        });
        return ResponseEntity.ok(iPage);
    }

    /**
     * 列表查询(苏州分布杨忆湄报销列表)
     * @param dto/reimbursementApproval/type/bxList
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list2")
    public ResponseEntity<IPage<RetureListDto>> list2(Page<RetureListDto> page, QueryListDto dto) throws Exception{
        IPage<RetureListDto> iPage = service.list2(page, dto);
        iPage.getRecords().stream().forEach(i -> {
            // 报销审批
            if (StringUtils.isNotBlank(i.getTaskId()) && StringUtils.isNotBlank(i.getType())) {
                if ("2".equals(i.getType())) { // “审核中”的数据要获取数据审核人
                    i.setAuditUserIds(flowTaskService.getTaskAuditUsers(i.getTaskId()));  // 获取数据审核人
                }
            }
        });
        return ResponseEntity.ok(iPage);
    }


    /**
     * 项目登记列表
     * @param info
     * @param page
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "项目登记列表")
    @GetMapping(value = "/projectList")
    public ResponseEntity<IPage<ProgramProjectListInfo>> projectList(ProgramProjectListInfo info, Page<ProgramProjectListInfo> page) throws Exception {
        IPage<ProgramProjectListInfo> list = service.projectList(page, info, "1");
        return ResponseEntity.ok (list);
    }

    /**
     * 新增/修改
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody SaveInfoDto dto) throws Exception{
        CwReimbursementInfo s = service.save(dto);
        return ResponseUtil.newInstance().add("businessTable", "cw_reimbursement_info")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .ok ("操作成功");
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @GetMapping("/remove")
    public ResponseEntity<String> remove(@RequestParam String id){
        String s = service.remove(id);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询
     * @param id
     * @return
     */
    @ApiOperation(value = "查询")
    @GetMapping("/findById")
    public ResponseEntity<SaveInfoDto> findById(@RequestParam String id) throws Exception{
        SaveInfoDto dto = service.findById(id);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "发票号判断")
    @GetMapping("/checkNumber")
    public ResponseEntity<Boolean> checkNumber(@RequestParam String number) {
        Boolean is = service.checkNumber(number);
        return ResponseEntity.ok(is);
    }

    /**
     * 根据id修改状态status
     */
    @ApiOperation(value = "根据id修改状态status")
    @PostMapping(value = "updateStatusById")
    public void updateStatusById(@RequestBody RetureListDto dto) {
        service.updateStatusById(dto);
    }

    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTree")
    public ResponseEntity<List<TreeUserDto>> userTree(@RequestParam String name) {
        List<TreeUserDto> list = service.userTree(name);
        return ResponseEntity.ok(list);
    }


    @ApiOperation(value = "根据租户ID查询用户树形")
    @GetMapping(value = "/userTreeByTenantId")
    public ResponseEntity<List<TreeUserDto>> userTreeByTenantId(@RequestParam String name) {
        List<TreeUserDto> list = service.userTreeByTenantId(name);
        return ResponseEntity.ok(list);
    }


    @ApiLog(value = "财务报销数据", type = LogTypeEnum.EXPORT)
    @GetMapping("exportFile")
    @ApiOperation(value = "财务报销数据")
    public void exportFile(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename();
        String sheetName = options.getSheetName();
        List<RetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            result = service.list (page,cwDTO).getRecords();
        }

        if(result.size()>0){
            //获取字典数据
            String statusDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("status");
            Map<String,Object> statusValueDTOs = JSON.parseObject(statusDatas, new TypeReference<Map<String,Object>>() {});

            String cwReimbursementSourceTypeDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("cw_reimbursement_source_type");
            Map<String,Object> cwReimbursementSourceTypeValueDTOs = JSON.parseObject(cwReimbursementSourceTypeDatas, new TypeReference<Map<String,Object>>() {});

            String paymentStatusDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("payment_status");
            Map<String,Object> paymentStatusValueDTOs = JSON.parseObject(paymentStatusDatas, new TypeReference<Map<String,Object>>() {});

            for (RetureListDto info : result) {

                if(StringUtils.isNotBlank(info.getType())){
                    for (String key : statusValueDTOs.keySet()) {
                        if(info.getType().equals(key)){
                            info.setType(String.valueOf(statusValueDTOs.get(key)));
                            break;
                        }
                    }
                }
                if(StringUtils.isNotBlank(info.getSourceType())){
                    for (String key : cwReimbursementSourceTypeValueDTOs.keySet()) {
                        if(info.getSourceType().equals(key)){
                            info.setSourceType(String.valueOf(cwReimbursementSourceTypeValueDTOs.get(key)));
                            break;
                        }
                    }
                }
                if(StringUtils.isNotBlank(info.getPaymentStatus())){
                    for (String key : paymentStatusValueDTOs.keySet()) {
                        if(info.getPaymentStatus().equals(key)){
                            info.setPaymentStatus(String.valueOf(paymentStatusValueDTOs.get(key)));
                            break;
                        }
                    }
                }else {
                    info.setPaymentStatus("未付款");
                }
            }
        }

        EasyPoiUtil.exportExcel ( result, sheetName,  sheetName, RetureListDto.class, fileName, response );
    }

    /**
     * 苏州报销(杨忆湄)
     * @param cwDTO
     * @param page
     * @param options
     * @param response
     * @throws Exception
     */
    @ApiLog(value = "财务报销数据", type = LogTypeEnum.EXPORT)
    @GetMapping("exportFile2")
    @ApiOperation(value = "财务报销数据")
    public void exportFile2(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename();
        String sheetName = options.getSheetName();
        List<RetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list2 (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list2 (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            result = service.list2 (page,cwDTO).getRecords();
        }
        EasyPoiUtil.exportExcel ( result, sheetName,  sheetName, RetureListDto.class, fileName, response );
    }


    @ApiLog(value = "财务电子发票报销数据")
    @GetMapping("exportInvoiceReimbursementFile")
    @ApiOperation(value = "财务电子发票报销数据")
    public void exportInvoiceReimbursementFile(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = "财务数电发票报销数据表";
        String sheetName = "财务数电发票报销列表";
        List<RetureListDto> result = new ArrayList<>();
        if ( com.jeeplus.common.excel.ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (com.jeeplus.common.excel.ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            if(com.jeeplus.utils.StringUtils.isBlank(cwDTO.getReimbursementType())){
                cwDTO.setReimbursementType("1");
            }
            result = service.list (page,cwDTO).getRecords();
        }
        List<String> idList = Lists.newArrayList();
        for (RetureListDto listDto : result) {
            idList.add(listDto.getId());
        }
        List<CwReimbursementAmountInfoDTO> invoiceReimbursementAmountList = Lists.newArrayList();
        if(idList.size()>0){
            invoiceReimbursementAmountList = service.getInvoiceReimbursementAmountList(idList);
        }
        for (int i = 1;i <=invoiceReimbursementAmountList.size();i ++){
            CwReimbursementAmountInfoDTO reimbursementAmountInfoDTO = invoiceReimbursementAmountList.get(i - 1);
            reimbursementAmountInfoDTO.setSerialNumber(i);
        }

        EasyPoiUtil.exportExcel ( invoiceReimbursementAmountList, sheetName,  sheetName, CwReimbursementAmountInfoDTO.class, fileName, response );

    }

    /**
     * 苏州报销(杨忆湄)
     * @param cwDTO
     * @param page
     * @param options
     * @param response
     * @throws Exception
     */
    @ApiLog(value = "财务电子发票报销数据")
    @GetMapping("exportInvoiceReimbursementFile2")
    @ApiOperation(value = "财务电子发票报销数据")
    public void exportInvoiceReimbursementFile2(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = "数电发票报销数据表";
        String sheetName = "数电发票报销列表";
        List<RetureListDto> result = new ArrayList<>();
        if ( com.jeeplus.common.excel.ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list2 (page,cwDTO).getRecords();
        } else if (com.jeeplus.common.excel.ExportMode.selected.equals ( options.getMode() )) {
            result = service.list2 (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            if(com.jeeplus.utils.StringUtils.isBlank(cwDTO.getReimbursementType())){
                cwDTO.setReimbursementType("1");
            }
            result = service.list2 (page,cwDTO).getRecords();
        }
        List<String> idList = Lists.newArrayList();
        for (RetureListDto listDto : result) {
            idList.add(listDto.getId());
        }
        List<CwReimbursementAmountInfoDTO> invoiceReimbursementAmountList = Lists.newArrayList();
        if(idList.size()>0){
            invoiceReimbursementAmountList = service.getInvoiceReimbursementAmountList(idList);
        }
        for (int i = 1;i <=invoiceReimbursementAmountList.size();i ++){
            CwReimbursementAmountInfoDTO reimbursementAmountInfoDTO = invoiceReimbursementAmountList.get(i - 1);
            reimbursementAmountInfoDTO.setSerialNumber(i);
        }

        EasyPoiUtil.exportExcel ( invoiceReimbursementAmountList, sheetName,  sheetName, CwReimbursementAmountInfoDTO.class, fileName, response );

    }


    @ApiOperation(value = "关联报销")
    @GetMapping(value = "/relationReimbursementList")
    public ResponseEntity<IPage<RetureListDto>> relationReimbursementList(Page<RetureListDto> page, @RequestParam("id") String id) {
        IPage<RetureListDto> listDtoIPage = service.relationReimbursementList(page, id);
        return ResponseEntity.ok(listDtoIPage);
    }

    @ApiOperation(value = "列表查询")
    @GetMapping("/queryByProIds")
    public ResponseEntity<List<CwProjectRecords>> queryByProIds(String ids){
        List<CwProjectRecords> cwProjectRecords = service.queryByProIds(ids);
        return ResponseEntity.ok(cwProjectRecords);
    }

    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByInvoiceNumber")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByInvoiceNumber(String invoiceNumber,String id){
        Map map = new HashMap();
        if(StringUtils.isBlank(invoiceNumber)){
            map.put("success",false);
            map.put("message","发票号不可为空");
        }else{
            Integer count = service.getEffectiveDataByInvoiceNumber(invoiceNumber,id);
            if(count>0){
                map.put("success",false);
                map.put("message","发票号：" + invoiceNumber + " 已发起或完成报销，请勿重复报销");
            }else{
                map.put("success",true);
            }
        }
        return map;
    }

    /**
     * 提交时根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByNumbers")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByNumbers(String invoiceNumber,String id){
        Map map = new HashMap();
        map.put("success",true);
        map.put("message","数电发票正常");
        if(StringUtils.isNotBlank(invoiceNumber)){
            String[] array = invoiceNumber.split(",");
            // 将数组转换为 List
            List<String> numberList = Arrays.asList(array);
            Set<String> set = new HashSet<>();
            if(numberList.size()>0){
                List<String> repetitionMessageList = service.getEffectiveDataByNumbers(numberList,id);
                for (String vatTax : repetitionMessageList) {
                    set.add(vatTax);
                }
            }
            ArrayList<String> strings = Lists.newArrayList(set);
            String result = String.join(",", strings);
            if(result.length()>0){
                map.put("success",false);
                map.put("message","数电发票编号为：" + result + "已经发起或已完成报销，无法重复报销");
            }

        }
        return map;
    }


    /**
     * 提交时根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @PostMapping("getEffectiveDataByNumbers2")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByNumbers2(@RequestBody QueryListDto dto){
        String invoiceNumber = dto.getFinanceNumber();
        String id = dto.getId();
        Map map = new HashMap();
        map.put("success",true);
        map.put("message","数电发票正常");
        if(StringUtils.isNotBlank(invoiceNumber)){
            String[] array = invoiceNumber.split(",");
            // 将数组转换为 List
            List<String> numberList = Arrays.asList(array);
            Set<String> set = new HashSet<>();
            if(numberList.size()>0){
                List<String> repetitionMessageList = service.getEffectiveDataByNumbers2(numberList, id);
                for (String vatTax : repetitionMessageList) {
                    set.add(vatTax);
                }
            }
            ArrayList<String> strings = Lists.newArrayList(set);
            String result = String.join(",", strings);
            if(result.length()>0){
                map.put("success",false);
                map.put("message","数电发票编号为：" + result + "已经发起或已完成报销，无法重复报销");
            }

        }
        return map;
    }

    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据报销id和发票编号删除数据记录")
    @PostMapping("deleteByIdAndNumber")
    @ApiOperation(value = "根据报销id和发票编号删除数据记录")
    public Map<String,Object> deleteByIdAndNumber(String id,String invoiceNumber){
        Map map = new HashMap();
        if(StringUtils.isBlank(invoiceNumber)){
            map.put("success",false);
            map.put("message","发票号不可为空");
        }else if(StringUtils.isBlank(id)){
            map.put("success",false);
            map.put("message","报销信息未选择");
        }else{
            Integer count = service.deleteByIdAndNumber(invoiceNumber,id);
            map.put("success",true);
        }
        return map;
    }

    /**
     * 查询有无历史数据
     * @param id
     * @return
     */
    @ApiOperation(value = "查询有无历史数据")
    @GetMapping("/findHistory")
    public ResponseEntity<List<CwReimbursementInfoUpHi>> findHistory(@RequestParam String id) throws Exception{
        List<CwReimbursementInfoUpHi> list=service.findHistory(id);
        return ResponseEntity.ok(list);
    }

    /**
     * 根据id查询历史数据
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询历史数据")
    @GetMapping("/findHiById")
    public ResponseEntity<SaveInfoDto> findHiById(@RequestParam String id) throws Exception{
        SaveInfoDto list=service.findHiById(id);
        return ResponseEntity.ok(list);
    }

    /**
     * 修改付款状态
     * @param
     * @return
     */
    @ApiOperation(value = "修改付款状态")
    @PostMapping("/updatePaymentStatus")
    public String updatePaymentStatus(@RequestBody RetureListDto dto) {
        String data=service.updatePaymentStatus(dto.getId(),dto.getPaymentStatus());
        return data;
    }


    /**
     * 查询暂时未归档报告归档数据
     */
    @ApiLog(value = "处理数电发票附件种购买方名称信息")
    @GetMapping("disposeElectronicEngineeringInvoice")
    @ApiOperation(value = "处理数电发票附件种购买方名称信息")
    public void disposeElectronicEngineeringInvoice(){
        service.disposeElectronicEngineeringInvoice();
    }

    /**
     * 查询文件补充
     * @param id
     * @return
     */
    @ApiOperation(value = "查询文件补充")
    @GetMapping("/findFileById")
    public ResponseEntity<SaveInfoDto> findFileById(@RequestParam String id) throws Exception{
        SaveInfoDto dto = service.findFileById(id);
        return ResponseEntity.ok(dto);
    }

    /**
     * 新增/修改 文件补充
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改 文件补充")
    @PostMapping("/saveFile")
    public ResponseEntity<String> saveFile(@RequestBody SaveInfoDto dto) throws Exception{
        CwReimbursementFileSupplement s = service.saveFile(dto);
        return ResponseUtil.newInstance().add("businessTable", "cw_reimbursement_file_supplement")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .add("name",s.getName())
                .ok ("操作成功");
    }

    /**
     * 根据id修改文件补充状态status
     */
    @ApiOperation(value = "根据id修改文件补充状态status")
    @PostMapping(value = "updateFileStatusById")
    public void updateFileStatusById(@RequestBody RetureListDto dto) {
        service.updateFileStatusById(dto);
    }



    /**
     * 根据项目ids查询项目名称
     * @return
     */
    @ApiLog("根据项目ids查询项目名称")
    @GetMapping("getCwProjectNameByIds")
    public String getCwProjectNameByIds(String ids) {
        String projectName = service.selectProjectByIds (ids);
        return JSON.toJSONString(projectName);
    }


    /**
     * 修改数电发票报销数据(对老数据进行调整)
     */
    @ApiOperation(value = "修改数电发票报销数据(对老数据进行调整)")
    @GetMapping(value = "updateOldData")
    public void updateOldData(){
        service.updateOldData();
    }


    /**
     * 查询文件补充(最新一条)
     * @param id
     * @return
     */
    @ApiOperation(value = "查询文件补充")
    @GetMapping("/findFileByReimId")
    public ResponseEntity<CwReimbursementFileSupplement> findFileByReimId(@RequestParam String id) throws Exception{
        CwReimbursementFileSupplement dto = service.findFileByReimId(id);
        return ResponseEntity.ok(dto);
    }


    /**
     * 离职查询用户是否存在电脑报销信息
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/computerList")
    public ResponseEntity<IPage<RetureListDto>> computerList(Page<RetureListDto> page, QueryListDto dto) throws Exception{
        IPage<RetureListDto> iPage = service.computerList(page, dto);
        iPage.getRecords().stream().forEach(i -> {
            // 报销审批
            if (StringUtils.isNotBlank(i.getTaskId()) && StringUtils.isNotBlank(i.getType())) {
                if ("2".equals(i.getType())) { // “审核中”的数据要获取数据审核人
                    i.setAuditUserIds(flowTaskService.getTaskAuditUsers(i.getTaskId()));  // 获取数据审核人
                    String currentTaskName = flowTaskService.getCurrentTaskName(i.getProcInsId());
                    if (com.jeeplus.utils.StringUtils.isNotBlank(currentTaskName) && !currentTaskName.contains("综合管理部主任") && !currentTaskName.contains("部门主任")){
                        i.setApplyFile(true);
                    }
                }
            }
            //文件补充
            if (StringUtils.isNotBlank(i.getTaskIdFile()) && StringUtils.isNotBlank(i.getFileStatus())) {
                if ("2".equals(i.getFileStatus()) || "4".equals(i.getFileStatus())) { // “审核中”的数据要获取数据审核人
                    i.setAuditFileUserIds(flowTaskService.getTaskAuditUsers(i.getTaskIdFile()));  // 获取数据审核人
                }
            }
        });
        return ResponseEntity.ok(iPage);
    }

}
