package com.jeeplus.human.depart.handover.controller;

import cn.hutool.extra.spring.SpringUtil;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.human.depart.handover.domain.HandoverRecovery;
import com.jeeplus.human.depart.handover.service.HandoverRecoveryService;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.logging.constant.enums.LogTypeEnum;
import com.jeeplus.sys.feign.IDictApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * @author 王强
 * @version 1.0
 * @date 2023-11-15 8:56
 */
@RestController
@Api("门卡回收")
@RequestMapping(value = "/handoverRecovery")
public class HandoverRecoveryController {

    @Resource
    private HandoverRecoveryService recoveryService;

    /**
     * 保存应收账款信息
     * @param handoverRecovery
     * @return
     */
    @ApiLog(value = "保存社保账款信息", type = LogTypeEnum.SAVE)
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody HandoverRecovery handoverRecovery) throws Exception {
        HandoverRecovery s = recoveryService.saveHandoverRecovery(handoverRecovery);
        return ResponseUtil.newInstance().add("businessTable", "human_resources_depart_handover_recovery")
                .add("businessId", s.getId())
                .ok ("操作成功");
    }

    /**
     * 查询
     * @param id
     * @return
     */
    @ApiOperation(value = "查询")
    @GetMapping("/findById")
    public ResponseEntity<HandoverRecovery> findById(@RequestParam String id) throws Exception{
        HandoverRecovery dto = recoveryService.findById(id);
        return ResponseEntity.ok(dto);
    }

    /**
     * 根据id修改状态status
     */
    @ApiOperation(value = "根据id修改状态status")
    @PostMapping(value = "updateStatusById")
    public void updateStatusById(@RequestBody HandoverRecovery dto) {
        recoveryService.updateStatusById(dto);
    }

    /**
     * 根据租户和用户数据删除钉钉用户数据
     */
    @ApiOperation(value = "根据租户和用户数据删除钉钉用户数据")
    @PostMapping(value = "deleteDingTalkByTenantId")
    public void deleteDingTalkByTenantId(String mobilePhone) {
        String dictValue = SpringUtil.getBean(IDictApi.class).getDictValue("dd_status", "dd_status", "0");
        if("1".equals(dictValue)) {
            recoveryService.deleteDingTalkByTenantId(mobilePhone);
        }
    }
}
