/**
 * Copyright &copy; 2021-2026 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.human.enrollment.enrollmentRegistration.utils;

import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.*;
import com.dingtalk.api.response.*;
import com.jeeplus.human.enrollment.enrollmentRegistration.service.dto.DingTalkUserDTO;

import com.taobao.api.ApiException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 钉钉工具类
 *
 * @author jeeplus
 * @version 2021-5-29
 */
@Component
public class DingTalkUtils {

    //会计钉钉配置
    @Value("${dingtalk.accountAppKey}")
    private String accountAppKey;

    @Value("${dingtalk.accountAppSecret}")
    private String accountAppSecret;

    //评估钉钉配置
    @Value("${dingtalk.assessAppKey}")
    private String assessAppKey;

    @Value("${dingtalk.assessAppSecret}")
    private String assessAppSecret;

    /**
     * 获取钉钉token
     */
    public  String getToken(String tenantId) throws ApiException {
        // 创建钉钉客户端
        DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/gettoken");
        // 配置请求参数
        OapiGettokenRequest req = new OapiGettokenRequest();
        String currentAppKey = "";
        String currentAppSecret = "";
        //根据公司判断来使用
        if("10003".equals(tenantId)){
            currentAppKey = accountAppKey;
            currentAppSecret = accountAppSecret;
        }else if("10001".equals(tenantId)){
            currentAppKey = assessAppKey;
            currentAppSecret = assessAppSecret;
        }
        req.setAppkey( currentAppKey); // 确保 appKey 已正确初始化
        req.setAppsecret(currentAppSecret); // 确保 appSecret 已正确初始化
        req.setHttpMethod("GET");
        // 执行请求，获取响应
        OapiGettokenResponse rsp = client.execute(req);
        String accessToken = rsp.getAccessToken();
        return accessToken;
    }
    

    /**
     * 获取部门列表
     */
    public  List<OapiV2DepartmentListsubResponse.DeptBaseResponse> deptList(String tenantId) throws ApiException {
        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> allDepartments = new ArrayList<>();
        Long rootDeptId = 1L;
        String token = getToken(tenantId);
        fetchAllDepartments(rootDeptId, allDepartments,token);
        return allDepartments;
    }

    // 假设有一个方法可以调用 API 获取子部门列表
    public  List<OapiV2DepartmentListsubResponse.DeptBaseResponse> getSubDepartments(Long parentDeptId,String token) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/department/listsub");
            OapiV2DepartmentListsubRequest req = new OapiV2DepartmentListsubRequest();
            // 检查 deptId 是否为空
            if (parentDeptId != null) {
                req.setDeptId(parentDeptId);
            }
            OapiV2DepartmentListsubResponse rsp = client.execute(req, token);
            List<OapiV2DepartmentListsubResponse.DeptBaseResponse> result = rsp.getResult();
            // 返回 JSON 响应
            return result;
        } catch (Exception e) {
            e.printStackTrace(); // 打印错误日志（开发调试用）
            return null;
        }
    }


    // 递归获取所有子部门
    public  void fetchAllDepartments(Long parentDeptId, List<OapiV2DepartmentListsubResponse.DeptBaseResponse> allDepts,String token) throws ApiException {
        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> subDepts = getSubDepartments(parentDeptId,token);
        for (OapiV2DepartmentListsubResponse.DeptBaseResponse dept : subDepts) {
            allDepts.add(dept); // 先添加当前部门
            fetchAllDepartments(dept.getDeptId(), allDepts,token); // 递归获取子部门
        }
    }


    /**
     * 创建用户
     */
    public  OapiV2UserCreateResponse addUser( DingTalkUserDTO addUserDTO,String tenantId) {
        try {
            String token = getToken(tenantId);
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/create");
            OapiV2UserCreateRequest req = new OapiV2UserCreateRequest();
            String name = addUserDTO.getName();
            String mobile = addUserDTO.getMobile();
            String deptIdList = addUserDTO.getDeptIdList();
            String userid = addUserDTO.getUserid();
            req.setName(name);
            req.setDeptIdList(deptIdList);
            req.setHideMobile(false);
            req.setMobile(mobile);
            req.setUserid(userid);
            OapiV2UserCreateResponse rsp = client.execute(req, token);
            // 返回 JSON 响应
            return rsp;
        } catch (Exception e) {
            throw  new RuntimeException(e.getMessage());
        }
    }

    public  OapiV2UserUpdateResponse updateUser(DingTalkUserDTO addUserDTO,String tenantId) {
        try {
            String token = getToken(tenantId);
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/update");
            OapiV2UserUpdateRequest req = new OapiV2UserUpdateRequest();
            String name = addUserDTO.getName();
            String mobile = addUserDTO.getMobile();
            String deptIdList = addUserDTO.getDeptIdList();
            String userId = addUserDTO.getUserid();
            req.setUserid(userId);
            req.setName(name);
            req.setDeptIdList(deptIdList);
            req.setHideMobile(false);
            req.setMobile(mobile);
            OapiV2UserUpdateResponse rsp = client.execute(req, token);
            // 返回 JSON 响应
            return rsp;
        } catch (Exception e) {
            throw  new RuntimeException(e.getMessage());
        }
    }

    /**
     * 获取用户信息
     * @param id
     * @return
     */
    public  OapiV2UserGetResponse.UserGetResponse selectUserById(String id,String tenantId) {
        try {
            String token = getToken(tenantId);
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/get");
            OapiV2UserGetRequest req = new OapiV2UserGetRequest();
            req.setUserid(id);
            req.setLanguage("zh_CN");
            OapiV2UserGetResponse rsp = client.execute(req, token);
            OapiV2UserGetResponse.UserGetResponse result = rsp.getResult();
            // 返回 JSON 响应
            return result;
        } catch (Exception e) {
            throw  new RuntimeException(e.getMessage());
        }
    }

    public  OapiV2UserDeleteResponse deleteUser(DingTalkUserDTO userDTO,String tenantId) {
        try {
            String token = getToken(tenantId);
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/delete");
            OapiV2UserDeleteRequest req = new OapiV2UserDeleteRequest();
            req.setUserid(userDTO.getUserid());
            OapiV2UserDeleteResponse rsp = client.execute(req, token);
            // 返回 JSON 响应
            return rsp;
        } catch (Exception e) {
            throw  new RuntimeException(e.getMessage());
        }
    }



}
